@@ -28,10 +28,6 @@ pcr.tune <- function(y, x, M = 10, maxk = 50, mat = NULL, ncores = 1, graph = TR
   M <- dim(mat)[2]
   rmat <- dim(mat)[1]
   msp <- matrix( nrow = M, ncol = maxk )
-  ## deigma will contain the positions of the test set
-  ## this is stored but not showed in the end
-  ## the user can access it though by running
-  ## the commands outside this function
   if (ncores == 1) {
 
     runtime <- proc.time()
@@ -42,26 +38,19 @@ pcr.tune <- function(y, x, M = 10, maxk = 50, mat = NULL, ncores = 1, graph = TR
       ytrain <- y[-mat[, vim] ]   ## train set dependent vars
       xtrain <- x[-mat[, vim], ]   ## train set independent vars
       xtest <- x[mat[, vim], , drop = FALSE]  ## test set independent vars
-
       m <- mean(ytrain)
       ytrain <- ytrain - m  ## standardize the dependent variable
-      mx <- Rfast::colmeans(xtrain)
-      s <- Rfast::colVars(xtrain, std = TRUE)
-      #mtrain <- t( xtrain )
-      #mtrain <- mtrain - mx
-      #mtrain <- t( mtrain / sqrt( Rfast::rowsums(mtrain^2) ) )
-      #sar <- tcrossprod( mtrain )
-      #eig <- eigen( sar )  ## eigen analysis of the design matrix
-      #vec <- eig$vectors  ## eigenvectors, or principal components
-	  vec <- prcomp(xtrain, center = TRUE, scale = TRUE)$rotation
+      vec <- prcomp(xtrain, center = TRUE, scale = TRUE)$rotation
       z <- xtrain %*% vec  ## PCA scores
-      xnew <- t( ( t(xtest) - mx ) / s ) ## standardize the xnew values
-
+      znew <- xtest %*% vec ## standardize the xnew values
+      zk <- Rfast::colsums(z^2)
+      zzk <- matrix(0, maxk, maxk)
+      diag(zzk) <- zk
+      cy <- crossprod( z, ytrain )
       for ( j in 1:maxk ) {
-        zzk <- crossprod(z[, 1:j])
-        be <- vec[, 1:j] %*% solve( zzk, crossprod( z[, 1:j], ytrain ) )
-        ## b is the PCA based coefficients
-        est <- as.vector( m + xnew %*% be )  ## predicted values for PCA model
+        zzkj <- zzk[1:j, 1:j]
+        be <- solve( zzkj, cy[1:j] )  ## (zzkj * zzkj^T )^(-1) * (z[1:j]^T * y)
+        est <- m + as.vector( znew[,1:j, drop = FALSE] %*% be )  ## predicted values for PCA model
         msp[vim, j] <- sum( (ytest - est)^2 ) / rmat
       }
     }
@@ -83,17 +72,17 @@ pcr.tune <- function(y, x, M = 10, maxk = 50, mat = NULL, ncores = 1, graph = TR
       xtest <-  x[mat[, vim], , drop = FALSE]  ## test set independent vars
       m <- mean(ytrain)
       ytrain <- ytrain - m  ## standardize the dependent variable
-      mx <- Rfast::colmeans(xtrain)
-      s <- Rfast::colVars(xtrain, std = TRUE)
- 	  vec <- prcomp(xtrain, center = TRUE, scale = TRUE)$rotation
+      vec <- prcomp(xtrain, center = TRUE, scale = TRUE)$rotation
       z <- xtrain %*% vec  ## PCA scores
-      xnew <- t( ( t(xtest) - mx ) / s ) ## standardize the xnew values
-
+      znew <- xtest %*% vec ## standardize the xnew values
+      zk <- Rfast::colsums(z^2)
+      zzk <- matrix(0, maxk, maxk)
+      diag(zzk) <- zk
+      cy <- crossprod( z, ytrain )
       for ( j in 1:maxk ) {
-        zzk <- crossprod(z[, 1:j])
-        be <- vec[, 1:j] %*% solve( zzk, crossprod( z[, 1:j], ytrain ) )
-        ## b is the PCA based coefficients
-        est <- as.vector( m + xnew %*% be )  ## predicted values for PCA model
+        zzkj <- zzk[1:j, 1:j]
+        be <- solve( zzkj, cy[1:j] )  ## (zzkj * zzkj^T )^(-1) * (z[1:j]^T * y)
+        est <- m + as.vector( znew %*% be )  ## predicted values for PCA model
         er[j] <- sum( (ytest - est)^2 ) / rmat
       }
       return(er)
@@ -104,12 +93,9 @@ pcr.tune <- function(y, x, M = 10, maxk = 50, mat = NULL, ncores = 1, graph = TR
   }
 
   mspe <- Rfast::colmeans(msp)
-  bias <- msp[, which.min(mspe)] - Rfast::rowMins(msp, value = TRUE)   ## apply(msp, 1, min)  ## TT estimate of bias
-  estb <- mean( bias )  ## TT estimate of bias
-
   if ( graph )  plot(1:maxk, mspe, xlab = "Number of principal components", ylab = "MSPE", type = "b")
   names(mspe) <- paste("PC", 1:maxk, sep = " ")
-  performance <- c( min(mspe) + estb, estb)
-  names(performance) <- c("MSPE", "Estimated bias")
+  performance <- min(mspe)
+  names(performance) <- "MSPE"
   list(msp = msp, mspe = mspe, k = which.min(mspe), performance = performance, runtime = runtime)
 }
